@@ -31,22 +31,22 @@
 #include "cif++/condition.hpp"
 #include "cif++/iterator.hpp"
 #include "cif++/row.hpp"
-#include "cif++/validate.hpp"
 #include "cif++/text.hpp"
+#include "cif++/validate.hpp"
 
 #include <array>
 
 /** \file category.hpp
-  * Documentation for the cif::category class
-  *
-  * The category class should meet the requirements of Container and
-  * SequenceContainer.
-  * 
-  * TODO: implement all of:
-  * https://en.cppreference.com/w/cpp/named_req/Container
-  * https://en.cppreference.com/w/cpp/named_req/SequenceContainer
-  * and more?
-  */
+ * Documentation for the cif::category class
+ *
+ * The category class should meet the requirements of Container and
+ * SequenceContainer.
+ *
+ * TODO: implement all of:
+ * https://en.cppreference.com/w/cpp/named_req/Container
+ * https://en.cppreference.com/w/cpp/named_req/SequenceContainer
+ * and more?
+ */
 
 namespace cif
 {
@@ -61,23 +61,43 @@ namespace cif
 class duplicate_key_error : public std::runtime_error
 {
   public:
-    /**
-     * @brief Construct a new duplicate key error object
-     */
+	/**
+	 * @brief Construct a new duplicate key error object
+	 */
 	duplicate_key_error(const std::string &msg)
 		: std::runtime_error(msg)
 	{
 	}
 };
 
+/// @brief A missing_key_error is thrown when an attempt is made
+/// to create an index when one of the key items is missing.
+class missing_key_error : public std::runtime_error
+{
+  public:
+	/**
+	 * @brief Construct a new duplicate key error object
+	 */
+	missing_key_error(const std::string &msg, const std::string &key)
+		: std::runtime_error(msg)
+		, m_key(key)
+	{
+	}
+
+	const std::string &get_key() const noexcept { return m_key; }
+
+  private:
+	std::string m_key;
+};
+
 /// @brief A multiple_results_error is throw when you request a single
 /// row using a query but the query contains more than exactly one row.
 class multiple_results_error : public std::runtime_error
 {
   public:
-    /**
-     * @brief Construct a new multiple results error object
-     */
+	/**
+	 * @brief Construct a new multiple results error object
+	 */
 	multiple_results_error()
 		: std::runtime_error("query should have returned exactly one row")
 	{
@@ -136,8 +156,16 @@ class category
 	// --------------------------------------------------------------------
 
 	const std::string &name() const { return m_name; } ///< Returns the name of the category
-	iset key_fields() const; ///< Returns the cif::iset of key field names. Retrieved from the @ref category_validator for this category
-	std::set<uint16_t> key_field_indices() const; ///< Returns a set of indices for the key fields.
+
+	[[deprecated("use key_items instead")]]
+	iset key_fields() const;                           ///< Returns the cif::iset of key item names. Retrieved from the @ref category_validator for this category
+
+	iset key_items() const;                           ///< Returns the cif::iset of key item names. Retrieved from the @ref category_validator for this category
+
+	[[deprecated("use key_item_indices instead")]]
+	std::set<uint16_t> key_field_indices() const;      ///< Returns a set of indices for the key items.
+
+	std::set<uint16_t> key_item_indices() const;      ///< Returns a set of indices for the key items.
 
 	/// @brief Set the validator for this category to @a v
 	/// @param v The category_validator to assign. A nullptr value is allowed.
@@ -162,7 +190,7 @@ class category
 
 	/// @brief Validate links, that means, values in this category should have an
 	/// accompanying value in parent categories.
-	/// 
+	///
 	/// @note
 	/// The code makes one exception when validating missing links and that's between
 	/// *atom_site* and a parent *pdbx_poly_seq_scheme* or *entity_poly_seq*.
@@ -265,7 +293,7 @@ class category
 	/// Return the theoretical maximum number or rows that can be stored
 	size_t max_size() const
 	{
-		return std::numeric_limits<size_t>::max();	// this is a bit optimistic, I guess
+		return std::numeric_limits<size_t>::max(); // this is a bit optimistic, I guess
 	}
 
 	/// Return true if the category is empty
@@ -281,12 +309,12 @@ class category
 	using key_type = row_initializer;
 
 	/// @brief Return a row_handle for the row specified by \a key
-	/// @param key The value for the key, fields specified in the dictionary should have a value
+	/// @param key The value for the key, items specified in the dictionary should have a value
 	/// @return The row found in the index, or an undefined row_handle
 	row_handle operator[](const key_type &key);
 
 	/// @brief Return a const row_handle for the row specified by \a key
-	/// @param key The value for the key, fields specified in the dictionary should have a value
+	/// @param key The value for the key, items specified in the dictionary should have a value
 	/// @return The row found in the index, or an undefined row_handle
 	const row_handle operator[](const key_type &key) const
 	{
@@ -301,15 +329,15 @@ class category
 	/// @code{.cpp}
 	/// for (const auto &[name, value] : cat.rows<std::string,int>("item_name", "item_value"))
 	///   std::cout << name << ": " << value << '\n';
-	/// @endcode 
+	/// @endcode
 	///
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 
 	template <typename... Ts, typename... Ns>
 	iterator_proxy<const category, Ts...> rows(Ns... names) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return iterator_proxy<const category, Ts...>(*this, begin(), { names... });
 	}
 
@@ -320,19 +348,19 @@ class category
 	/// for (const auto &[name, value] : cat.rows<std::string,int>("item_name", "item_value"))
 	///   std::cout << name << ": " << value << '\n';
 	///
-	/// // or in case we only need one column:
+	/// // or in case we only need one item:
 	///
 	/// for (int id : cat.rows<int>("id"))
 	///   std::cout << id << '\n';
-	/// @endcode 
+	/// @endcode
 	///
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 
 	template <typename... Ts, typename... Ns>
 	iterator_proxy<category, Ts...> rows(Ns... names)
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return iterator_proxy<category, Ts...>(*this, begin(), { names... });
 	}
 
@@ -343,7 +371,7 @@ class category
 	/// @code{.cpp}
 	/// for (row_handle rh : cat.find(cif::key("first_name") == "John" and cif::key("last_name") == "Doe"))
 	///    .. // do something with rh
-	/// @endcode 
+	/// @endcode
 	///
 	/// @param cond The condition for the query
 	/// @return A special iterator that loops over all elements that match. The iterator can be dereferenced
@@ -397,17 +425,17 @@ class category
 	/// @code{.cpp}
 	/// for (const auto &[name, value] : cat.find<std::string,int>(cif::key("item_value") > 10, "item_name", "item_value"))
 	///    std::cout << name << ": " << value << '\n';
-	/// @endcode 
+	/// @endcode
 	///
 	/// @param cond The condition for the query
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 	/// @return A special iterator that loops over all elements that match.
 
 	template <typename... Ts, typename... Ns>
 	conditional_iterator_proxy<category, Ts...> find(condition &&cond, Ns... names)
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return find<Ts...>(cbegin(), std::move(cond), std::forward<Ns>(names)...);
 	}
 
@@ -415,14 +443,14 @@ class category
 	/// iterator can be used in a structured binding context.
 	///
 	/// @param cond The condition for the query
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 	/// @return A special iterator that loops over all elements that match.
 
 	template <typename... Ts, typename... Ns>
 	conditional_iterator_proxy<const category, Ts...> find(condition &&cond, Ns... names) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return find<Ts...>(cbegin(), std::move(cond), std::forward<Ns>(names)...);
 	}
 
@@ -431,14 +459,14 @@ class category
 	///
 	/// @param pos Iterator pointing to the location where to start
 	/// @param cond The condition for the query
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 	/// @return A special iterator that loops over all elements that match.
 
 	template <typename... Ts, typename... Ns>
 	conditional_iterator_proxy<category, Ts...> find(const_iterator pos, condition &&cond, Ns... names)
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return { *this, pos, std::move(cond), std::forward<Ns>(names)... };
 	}
 
@@ -447,14 +475,14 @@ class category
 	///
 	/// @param pos Iterator pointing to the location where to start
 	/// @param cond The condition for the query
-	/// @tparam Ts The types for the columns requested
-	/// @param names The names for the columns requested
+	/// @tparam Ts The types for the items requested
+	/// @param names The names for the items requested
 	/// @return A special iterator that loops over all elements that match.
 
 	template <typename... Ts, typename... Ns>
 	conditional_iterator_proxy<const category, Ts...> find(const_iterator pos, condition &&cond, Ns... names) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of column titles should be equal to the number of types to return");
+		static_assert(sizeof...(Ts) == sizeof...(Ns), "The number of item names should be equal to the number of types to return");
 		return { *this, pos, std::move(cond), std::forward<Ns>(names)... };
 	}
 
@@ -509,30 +537,30 @@ class category
 		return *h.begin();
 	}
 
-	/// @brief Return value for the column named @a column for the single row that
+	/// @brief Return value for the item named @a item for the single row that
 	/// matches @a cond. Throws @a multiple_results_error if there are is not exactly one row
 	/// @tparam The type to use for the result
 	/// @param cond The condition to search for
-	/// @param column The name of the column to return the value for
+	/// @param item The name of the item to return the value for
 	/// @return The value found
 	template <typename T>
-	T find1(condition &&cond, const char *column) const
+	T find1(condition &&cond, std::string_view item) const
 	{
-		return find1<T>(cbegin(), std::move(cond), column);
+		return find1<T>(cbegin(), std::move(cond), item);
 	}
 
-	/// @brief Return value for the column named @a column for the single row that
+	/// @brief Return value for the item named @a item for the single row that
 	/// matches @a cond when starting to search at @a pos.
 	/// Throws @a multiple_results_error if there are is not exactly one row
 	/// @tparam The type to use for the result
 	/// @param pos The location to start the search
 	/// @param cond The condition to search for
-	/// @param column The name of the column to return the value for
+	/// @param item The name of the item to return the value for
 	/// @return The value found
 	template <typename T, std::enable_if_t<not is_optional_v<T>, int> = 0>
-	T find1(const_iterator pos, condition &&cond, const char *column) const
+	T find1(const_iterator pos, condition &&cond, std::string_view item) const
 	{
-		auto h = find<T>(pos, std::move(cond), column);
+		auto h = find<T>(pos, std::move(cond), item);
 
 		if (h.size() != 1)
 			throw multiple_results_error();
@@ -540,18 +568,18 @@ class category
 		return *h.begin();
 	}
 
-	/// @brief Return a value of type std::optional<T> for the column named @a column for the single row that
+	/// @brief Return a value of type std::optional<T> for the item named @a item for the single row that
 	/// matches @a cond when starting to search at @a pos.
 	/// If the row was not found, an empty value is returned.
 	/// @tparam The type to use for the result
 	/// @param pos The location to start the search
 	/// @param cond The condition to search for
-	/// @param column The name of the column to return the value for
+	/// @param item The name of the item to return the value for
 	/// @return The value found, can be empty if no row matches the condition
 	template <typename T, std::enable_if_t<is_optional_v<T>, int> = 0>
-	T find1(const_iterator pos, condition &&cond, const char *column) const
+	T find1(const_iterator pos, condition &&cond, std::string_view item) const
 	{
-		auto h = find<typename T::value_type>(pos, std::move(cond), column);
+		auto h = find<typename T::value_type>(pos, std::move(cond), item);
 
 		if (h.size() > 1)
 			throw multiple_results_error();
@@ -562,34 +590,34 @@ class category
 		return *h.begin();
 	}
 
-	/// @brief Return a std::tuple for the values for the columns named in @a columns
+	/// @brief Return a std::tuple for the values for the items named in @a items
 	/// for the single row that matches @a cond
 	/// Throws @a multiple_results_error if there are is not exactly one row
 	/// @tparam The types to use for the resulting tuple
 	/// @param cond The condition to search for
-	/// @param columns The names of the columns to return the value for
+	/// @param items The names of the items to return the value for
 	/// @return The values found as a single tuple of type std::tuple<Ts...>
 	template <typename... Ts, typename... Cs, typename U = std::enable_if_t<sizeof...(Ts) != 1>>
-	std::tuple<Ts...> find1(condition &&cond, Cs... columns) const
+	std::tuple<Ts...> find1(condition &&cond, Cs... items) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of column titles should be equal to the number of types to return");
-		// static_assert(std::is_same_v<Cs, const char*>..., "The column names should be const char");
-		return find1<Ts...>(cbegin(), std::move(cond), std::forward<Cs>(columns)...);
+		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of item names should be equal to the number of types to return");
+		// static_assert(std::is_same_v<Cs, const char*>..., "The item names should be const char");
+		return find1<Ts...>(cbegin(), std::move(cond), std::forward<Cs>(items)...);
 	}
 
-	/// @brief Return a std::tuple for the values for the columns named in @a columns
+	/// @brief Return a std::tuple for the values for the items named in @a items
 	/// for the single row that matches @a cond when starting to search at @a pos
 	/// Throws @a multiple_results_error if there are is not exactly one row
 	/// @tparam The types to use for the resulting tuple
 	/// @param pos The location to start the search
 	/// @param cond The condition to search for
-	/// @param columns The names of the columns to return the value for
+	/// @param items The names of the items to return the value for
 	/// @return The values found as a single tuple of type std::tuple<Ts...>
 	template <typename... Ts, typename... Cs, typename U = std::enable_if_t<sizeof...(Ts) != 1>>
-	std::tuple<Ts...> find1(const_iterator pos, condition &&cond, Cs... columns) const
+	std::tuple<Ts...> find1(const_iterator pos, condition &&cond, Cs... items) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of column titles should be equal to the number of types to return");
-		auto h = find<Ts...>(pos, std::move(cond), std::forward<Cs>(columns)...);
+		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of item names should be equal to the number of types to return");
+		auto h = find<Ts...>(pos, std::move(cond), std::forward<Cs>(items)...);
 
 		if (h.size() != 1)
 			throw multiple_results_error();
@@ -638,74 +666,74 @@ class category
 		return h.empty() ? row_handle{} : *h.begin();
 	}
 
-	/// @brief Return the value for column @a column for the first row that matches condition @a cond
+	/// @brief Return the value for item @a item for the first row that matches condition @a cond
 	/// @tparam The type of the value to return
 	/// @param cond The condition to search for
-	/// @param column The column for which the value should be returned
+	/// @param item The item for which the value should be returned
 	/// @return The value found or a default constructed value if not found
 	template <typename T>
-	T find_first(condition &&cond, const char *column) const
+	T find_first(condition &&cond, std::string_view item) const
 	{
-		return find_first<T>(cbegin(), std::move(cond), column);
+		return find_first<T>(cbegin(), std::move(cond), item);
 	}
 
-	/// @brief Return the value for column @a column for the first row that matches condition @a cond
+	/// @brief Return the value for item @a item for the first row that matches condition @a cond
 	/// when starting the search at @a pos
 	/// @tparam The type of the value to return
 	/// @param pos The location to start searching
 	/// @param cond The condition to search for
-	/// @param column The column for which the value should be returned
+	/// @param item The item for which the value should be returned
 	/// @return The value found or a default constructed value if not found
 	template <typename T>
-	T find_first(const_iterator pos, condition &&cond, const char *column) const
+	T find_first(const_iterator pos, condition &&cond, std::string_view item) const
 	{
-		auto h = find<T>(pos, std::move(cond), column);
+		auto h = find<T>(pos, std::move(cond), item);
 
 		return h.empty() ? T{} : *h.begin();
 	}
 
-	/// @brief Return a tuple containing the values for the columns @a columns for the first row that matches condition @a cond
+	/// @brief Return a tuple containing the values for the items @a items for the first row that matches condition @a cond
 	/// @tparam The types of the values to return
 	/// @param cond The condition to search for
-	/// @param columns The columns for which the values should be returned
+	/// @param items The items for which the values should be returned
 	/// @return The values found or default constructed values if not found
 	template <typename... Ts, typename... Cs, typename U = std::enable_if_t<sizeof...(Ts) != 1>>
-	std::tuple<Ts...> find_first(condition &&cond, Cs... columns) const
+	std::tuple<Ts...> find_first(condition &&cond, Cs... items) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of column titles should be equal to the number of types to return");
-		// static_assert(std::is_same_v<Cs, const char*>..., "The column names should be const char");
-		return find_first<Ts...>(cbegin(), std::move(cond), std::forward<Cs>(columns)...);
+		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of item names should be equal to the number of types to return");
+		// static_assert(std::is_same_v<Cs, const char*>..., "The item names should be const char");
+		return find_first<Ts...>(cbegin(), std::move(cond), std::forward<Cs>(items)...);
 	}
 
-	/// @brief Return a tuple containing the values for the columns @a columns for the first row that matches condition @a cond
+	/// @brief Return a tuple containing the values for the items @a items for the first row that matches condition @a cond
 	/// when starting the search at @a pos
 	/// @tparam The types of the values to return
 	/// @param pos The location to start searching
 	/// @param cond The condition to search for
-	/// @param columns The columns for which the values should be returned
+	/// @param items The items for which the values should be returned
 	/// @return The values found or default constructed values if not found
 	template <typename... Ts, typename... Cs, typename U = std::enable_if_t<sizeof...(Ts) != 1>>
-	std::tuple<Ts...> find_first(const_iterator pos, condition &&cond, Cs... columns) const
+	std::tuple<Ts...> find_first(const_iterator pos, condition &&cond, Cs... items) const
 	{
-		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of column titles should be equal to the number of types to return");
-		auto h = find<Ts...>(pos, std::move(cond), std::forward<Cs>(columns)...);
+		static_assert(sizeof...(Ts) == sizeof...(Cs), "The number of item names should be equal to the number of types to return");
+		auto h = find<Ts...>(pos, std::move(cond), std::forward<Cs>(items)...);
 
 		return h.empty() ? std::tuple<Ts...>{} : *h.begin();
 	}
 
 	// --------------------------------------------------------------------
 
-	/// @brief Return the maximum value for column @a column for all rows that match condition @a cond
+	/// @brief Return the maximum value for item @a item for all rows that match condition @a cond
 	/// @tparam The type of the value to return
-	/// @param column The column to use for the value
+	/// @param item The item to use for the value
 	/// @param cond The condition to search for
 	/// @return The value found or the minimal value for the type
 	template <typename T, std::enable_if_t<std::is_arithmetic_v<T>, int> = 0>
-	T find_max(const char *column, condition &&cond) const
+	T find_max(std::string_view item, condition &&cond) const
 	{
 		T result = std::numeric_limits<T>::min();
 
-		for (auto v : find<T>(std::move(cond), column))
+		for (auto v : find<T>(std::move(cond), item))
 		{
 			if (result < v)
 				result = v;
@@ -714,27 +742,27 @@ class category
 		return result;
 	}
 
-	/// @brief Return the maximum value for column @a column for all rows
+	/// @brief Return the maximum value for item @a item for all rows
 	/// @tparam The type of the value to return
-	/// @param column The column to use for the value
+	/// @param item The item to use for the value
 	/// @return The value found or the minimal value for the type
 	template <typename T, std::enable_if_t<std::is_arithmetic_v<T>, int> = 0>
-	T find_max(const char *column) const
+	T find_max(std::string_view item) const
 	{
-		return find_max<T>(column, all());
+		return find_max<T>(item, all());
 	}
 
-	/// @brief Return the minimum value for column @a column for all rows that match condition @a cond
+	/// @brief Return the minimum value for item @a item for all rows that match condition @a cond
 	/// @tparam The type of the value to return
-	/// @param column The column to use for the value
+	/// @param item The item to use for the value
 	/// @param cond The condition to search for
 	/// @return The value found or the maximum value for the type
 	template <typename T, std::enable_if_t<std::is_arithmetic_v<T>, int> = 0>
-	T find_min(const char *column, condition &&cond) const
+	T find_min(std::string_view item, condition &&cond) const
 	{
 		T result = std::numeric_limits<T>::max();
 
-		for (auto v : find<T>(std::move(cond), column))
+		for (auto v : find<T>(std::move(cond), item))
 		{
 			if (result > v)
 				result = v;
@@ -743,20 +771,28 @@ class category
 		return result;
 	}
 
-	/// @brief Return the maximum value for column @a column for all rows
+	/// @brief Return the maximum value for item @a item for all rows
 	/// @tparam The type of the value to return
-	/// @param column The column to use for the value
+	/// @param item The item to use for the value
 	/// @return The value found or the maximum value for the type
 	template <typename T, std::enable_if_t<std::is_arithmetic_v<T>, int> = 0>
-	T find_min(const char *column) const
+	T find_min(std::string_view item) const
 	{
-		return find_min<T>(column, all());
+		return find_min<T>(item, all());
 	}
 
 	/// @brief Return whether a row exists that matches condition @a cond
 	/// @param cond The condition to match
 	/// @return True if a row exists
-	bool exists(condition &&cond) const
+	[[deprecated("Use contains instead")]] bool exists(condition &&cond) const
+	{
+		return contains(std::move(cond));
+	}
+
+	/// @brief Return whether a row exists that matches condition @a cond
+	/// @param cond The condition to match
+	/// @return True if a row exists
+	bool contains(condition &&cond) const
 	{
 		bool result = false;
 
@@ -846,7 +882,7 @@ class category
 	// 	insert_impl(pos, std::move(row));
 	// }
 
-	/// Erase the row pointed to by @a pos and return the iterator to the 
+	/// Erase the row pointed to by @a pos and return the iterator to the
 	/// row following pos.
 	iterator erase(iterator pos);
 
@@ -890,7 +926,7 @@ class category
 			for (auto i = b; i != e; ++i)
 			{
 				// item_value *new_item = this->create_item(*i);
-				r->append(add_column(i->name()), { i->value() });
+				r->append(add_item(i->name()), { i->value() });
 			}
 		}
 		catch (...)
@@ -912,7 +948,6 @@ class category
 	/// result is unique in the context of this category
 	std::string get_unique_id(std::function<std::string(int)> generator = cif::cif_id_for_number);
 
-
 	/// @brief Generate a new, unique ID based on a string prefix followed by a number
 	/// @param prefix The string prefix
 	/// @return a new unique ID
@@ -922,98 +957,201 @@ class category
 			{ return prefix + std::to_string(nr + 1); });
 	}
 
+	/// @brief Generate a new, unique value for a item named @a item_name
+	/// @param item_name The name of the item
+	/// @return a new unique value
+	std::string get_unique_value(std::string_view item_name);
+
 	// --------------------------------------------------------------------
 
-	/// \brief Update a single column named @a tag in the rows that match \a cond to value \a value
+	using value_provider_type = std::function<std::string_view(std::string_view)>;
+
+	/// \brief Update a single item named @a item_name in the rows that match
+	/// \a cond to values provided by a callback function \a value_provider
 	/// making sure the linked categories are updated according to the link.
 	/// That means, child categories are updated if the links are absolute
 	/// and unique. If they are not, the child category rows are split.
 
-	void update_value(condition &&cond, std::string_view tag, std::string_view value)
+	void update_value(condition &&cond, std::string_view item_name,
+		value_provider_type &&value_provider)
 	{
 		auto rs = find(std::move(cond));
 		std::vector<row_handle> rows;
 		std::copy(rs.begin(), rs.end(), std::back_inserter(rows));
-		update_value(rows, tag, value);
+		update_value(rows, item_name, std::move(value_provider));
 	}
 
-	/// \brief Update a single column named @a tag in @a rows to value \a value
+	/// \brief Update a single item named @a item_name in the rows \a rows
+	/// to values provided by a callback function \a value_provider
 	/// making sure the linked categories are updated according to the link.
 	/// That means, child categories are updated if the links are absolute
 	/// and unique. If they are not, the child category rows are split.
 
-	void update_value(const std::vector<row_handle> &rows, std::string_view tag, std::string_view value);
+	void update_value(const std::vector<row_handle> &rows, std::string_view item_name,
+		value_provider_type &&value_provider);
+
+	/// \brief Update a single item named @a item_name in the rows that match \a cond to value \a value
+	/// making sure the linked categories are updated according to the link.
+	/// That means, child categories are updated if the links are absolute
+	/// and unique. If they are not, the child category rows are split.
+
+	void update_value(condition &&cond, std::string_view item_name, std::string_view value)
+	{
+		auto rs = find(std::move(cond));
+		std::vector<row_handle> rows;
+		std::copy(rs.begin(), rs.end(), std::back_inserter(rows));
+		update_value(rows, item_name, value);
+	}
+
+	/// \brief Update a single item named @a item_name in @a rows to value \a value
+	/// making sure the linked categories are updated according to the link.
+	/// That means, child categories are updated if the links are absolute
+	/// and unique. If they are not, the child category rows are split.
+
+	void update_value(const std::vector<row_handle> &rows, std::string_view item_name, std::string_view value)
+	{
+		update_value(rows, item_name, [value](std::string_view) { return value; });
+	}
 
 	// --------------------------------------------------------------------
-	/// \brief Return the index number for \a column_name
+	// Naming used to be very inconsistent. For backward compatibility,
+	// the old function names are here as deprecated variants.
 
+	/// \brief Return the index number for \a column_name
+	[[deprecated("Use get_item_ix instead")]]
 	uint16_t get_column_ix(std::string_view column_name) const
 	{
-		uint16_t result;
-
-		for (result = 0; result < m_columns.size(); ++result)
-		{
-			if (iequals(column_name, m_columns[result].m_name))
-				break;
-		}
-
-		if (VERBOSE > 0 and result == m_columns.size() and m_cat_validator != nullptr) // validate the name, if it is known at all (since it was not found)
-		{
-			auto iv = m_cat_validator->get_validator_for_item(column_name);
-			if (iv == nullptr)
-				std::cerr << "Invalid name used '" << column_name << "' is not a known column in " + m_name << '\n';
-		}
-
-		return result;
+		return get_item_ix(column_name);
 	}
 
 	/// @brief Return the name for column with index @a ix
 	/// @param ix The index number
 	/// @return The name of the column
+	[[deprecated("use get_item_name instead")]]
 	std::string_view get_column_name(uint16_t ix) const
 	{
-		if (ix >= m_columns.size())
-			throw std::out_of_range("column index is out of range");
-
-		return m_columns[ix].m_name;
+		return get_item_name(ix);
 	}
 
-	/// @brief Make sure a column with name @a column_name is known and return its index number
-	/// @param column_name The name of the column
-	/// @return The index number of the column
-	uint16_t add_column(std::string_view column_name)
+	/// @brief Make sure a item with name @a item_name is known and return its index number
+	/// @param item_name The name of the item
+	/// @return The index number of the item
+	[[deprecated("use add_item instead")]]
+	uint16_t add_column(std::string_view item_name)
 	{
-		using namespace std::literals;
+		return add_item(item_name);
+	}
 
-		uint16_t result = get_column_ix(column_name);
+	/** @brief Remove column name @a colum_name
+	 * @param column_name The column to be removed
+	 */
+	[[deprecated("use remove_item instead")]]
+	void remove_column(std::string_view column_name)
+	{
+		remove_item(column_name);
+	}
 
-		if (result == m_columns.size())
-		{
-			const item_validator *item_validator = nullptr;
-
-			if (m_cat_validator != nullptr)
-			{
-				item_validator = m_cat_validator->get_validator_for_item(column_name);
-				if (item_validator == nullptr)
-					m_validator->report_error("tag " + std::string(column_name) + " not allowed in category " + m_name, false);
-			}
-
-			m_columns.emplace_back(column_name, item_validator);
-		}
-
-		return result;
+	/** @brief Rename column @a from_name to @a to_name */
+	[[deprecated("use rename_item instead")]]
+	void rename_column(std::string_view from_name, std::string_view to_name)
+	{
+		rename_item(from_name, to_name);
 	}
 
 	/// @brief Return whether a column with name @a name exists in this category
 	/// @param name The name of the column
 	/// @return True if the column exists
+	[[deprecated("use has_item instead")]]
 	bool has_column(std::string_view name) const
 	{
-		return get_column_ix(name) < m_columns.size();
+		return has_item(name);
 	}
 
 	/// @brief Return the cif::iset of columns in this category
-	iset get_columns() const;
+	[[deprecated("use get_items instead")]]
+	iset get_columns() const
+	{
+		return get_items();
+	}
+
+	// --------------------------------------------------------------------
+	/// \brief Return the index number for \a item_name
+
+	uint16_t get_item_ix(std::string_view item_name) const
+	{
+		uint16_t result;
+
+		for (result = 0; result < m_items.size(); ++result)
+		{
+			if (iequals(item_name, m_items[result].m_name))
+				break;
+		}
+
+		if (VERBOSE > 0 and result == m_items.size() and m_cat_validator != nullptr) // validate the name, if it is known at all (since it was not found)
+		{
+			auto iv = m_cat_validator->get_validator_for_item(item_name);
+			if (iv == nullptr)
+				std::cerr << "Invalid name used '" << item_name << "' is not a known item in " + m_name << '\n';
+		}
+
+		return result;
+	}
+
+	/// @brief Return the name for item with index @a ix
+	/// @param ix The index number
+	/// @return The name of the item
+	std::string_view get_item_name(uint16_t ix) const
+	{
+		if (ix >= m_items.size())
+			throw std::out_of_range("item index is out of range");
+
+		return m_items[ix].m_name;
+	}
+
+	/// @brief Make sure a item with name @a item_name is known and return its index number
+	/// @param item_name The name of the item
+	/// @return The index number of the item
+	uint16_t add_item(std::string_view item_name)
+	{
+		using namespace std::literals;
+
+		uint16_t result = get_item_ix(item_name);
+
+		if (result == m_items.size())
+		{
+			const item_validator *item_validator = nullptr;
+
+			if (m_cat_validator != nullptr)
+			{
+				item_validator = m_cat_validator->get_validator_for_item(item_name);
+				if (item_validator == nullptr)
+					m_validator->report_error( validation_error::item_not_allowed_in_category, m_name, item_name, false);
+			}
+
+			m_items.emplace_back(item_name, item_validator);
+		}
+
+		return result;
+	}
+
+	/** @brief Remove item name @a colum_name
+	 * @param item_name The item to be removed
+	 */
+	void remove_item(std::string_view item_name);
+
+	/** @brief Rename item @a from_name to @a to_name */
+	void rename_item(std::string_view from_name, std::string_view to_name);
+
+	/// @brief Return whether a item with name @a name exists in this category
+	/// @param name The name of the item
+	/// @return True if the item exists
+	bool has_item(std::string_view name) const
+	{
+		return get_item_ix(name) < m_items.size();
+	}
+
+	/// @brief Return the cif::iset of items in this category
+	iset get_items() const;
 
 	// --------------------------------------------------------------------
 
@@ -1029,30 +1167,37 @@ class category
 
 	// --------------------------------------------------------------------
 
-	/// This function returns effectively the list of fully qualified column
-	/// names, that is category_name + '.' + column_name for each column
-	std::vector<std::string> get_tag_order() const;
+	/// This function returns effectively the list of fully qualified item
+	/// names, that is category_name + '.' + item_name for each item
+	[[deprecated("use get_item_order instead")]]
+	std::vector<std::string> get_tag_order() const
+	{
+		return get_item_order();
+	}
+
+	/// This function returns effectively the list of fully qualified item
+	/// names, that is category_name + '.' + item_name for each item
+	std::vector<std::string> get_item_order() const;
 
 	/// Write the contents of the category to the std::ostream @a os
 	void write(std::ostream &os) const;
 
 	/// @brief Write the contents of the category to the std::ostream @a os and
-	/// use @a order as the order of the columns. If @a addMissingColumns is
-	/// false, columns that do not contain any value will be suppressed
+	/// use @a order as the order of the items. If @a addMissingItems is
+	/// false, items that do not contain any value will be suppressed
 	/// @param os The std::ostream to write to
-	/// @param order The order in which the columns should appear
-	/// @param addMissingColumns When false, empty columns are suppressed from the output
-	void write(std::ostream &os, const std::vector<std::string> &order, bool addMissingColumns = true);
+	/// @param order The order in which the items should appear
+	/// @param addMissingItems When false, empty items are suppressed from the output
+	void write(std::ostream &os, const std::vector<std::string> &order, bool addMissingItems = true);
 
   private:
-	void write(std::ostream &os, const std::vector<uint16_t> &order, bool includeEmptyColumns) const;
+	void write(std::ostream &os, const std::vector<uint16_t> &order, bool includeEmptyItems) const;
 
   public:
-
 	/// friend function to make it possible to do:
 	/// @code {.cpp}
 	/// std::cout << my_category;
-	/// @endcode 
+	/// @endcode
 	friend std::ostream &operator<<(std::ostream &os, const category &cat)
 	{
 		cat.write(os);
@@ -1060,7 +1205,7 @@ class category
 	}
 
   private:
-	void update_value(row *row, uint16_t column, std::string_view value, bool updateLinked, bool validate = true);
+	void update_value(row *row, uint16_t item, std::string_view value, bool updateLinked, bool validate = true);
 
 	void erase_orphans(condition &&cond, category &parent);
 
@@ -1097,12 +1242,12 @@ class category
 
 	row_handle create_copy(row_handle r);
 
-	struct item_column
+	struct item_entry
 	{
 		std::string m_name;
 		const item_validator *m_validator;
 
-		item_column(std::string_view name, const item_validator *validator)
+		item_entry(std::string_view name, const item_validator *validator)
 			: m_name(name)
 			, m_validator(validator)
 		{
@@ -1132,12 +1277,12 @@ class category
 
 	// --------------------------------------------------------------------
 
-	void swap_item(uint16_t column_ix, row_handle &a, row_handle &b);
+	void swap_item(uint16_t item_ix, row_handle &a, row_handle &b);
 
 	// --------------------------------------------------------------------
 
 	std::string m_name;
-	std::vector<item_column> m_columns;
+	std::vector<item_entry> m_items;
 	const validator *m_validator = nullptr;
 	const category_validator *m_cat_validator = nullptr;
 	std::vector<link> m_parent_links, m_child_links;
