@@ -1493,8 +1493,8 @@ bool Remark3Parser::parse(const std::string &expMethod, PDBRecord *r, cif::datab
 					auto r1 = cat1.front();
 					auto r2 = cat2.front();
 
-					for (auto column : cat1.key_fields())
-						r2[column] = r1[column].text();
+					for (auto item : cat1.key_items())
+						r2[item] = r1[item].text();
 				}
 			}
 			else
