@@ -39,16 +39,33 @@
 // the code will use boost::regex instead.
 
 #if USE_BOOST_REGEX
-#include <boost/regex.hpp>
+# include <boost/regex.hpp>
 using boost::regex;
 #else
-#include <regex>
+# include <regex>
 using std::regex;
 #endif
 
 namespace cif
 {
 
+validation_exception::validation_exception(std::error_code ec)
+	: runtime_error(ec.message())
+{
+}
+
+validation_exception::validation_exception(std::error_code ec, std::string_view category)
+	: runtime_error((ec.message() + "; category: ").append(category))
+{
+}
+
+validation_exception::validation_exception(std::error_code ec, std::string_view category, std::string_view item)
+	: runtime_error((ec.message() + "; category: ").append(category).append("; item: ").append(item))
+{
+}
+
+// --------------------------------------------------------------------
+
 struct regex_impl : public regex
 {
 	regex_impl(std::string_view rx)
@@ -57,21 +74,12 @@ struct regex_impl : public regex
 	}
 };
 
-validation_error::validation_error(const std::string &msg)
-	: m_msg(msg)
-{
-}
-
-validation_error::validation_error(const std::string &cat, const std::string &item, const std::string &msg)
-	: m_msg("When validating _" + cat + '.' + item + ": " + msg)
-{
-}
-
 // --------------------------------------------------------------------
 
-DDL_PrimitiveType map_to_primitive_type(std::string_view s)
+DDL_PrimitiveType map_to_primitive_type(std::string_view s, std::error_code &ec) noexcept
 {
-	DDL_PrimitiveType result;
+	ec = {};
+	DDL_PrimitiveType result = DDL_PrimitiveType::Char;
 	if (iequals(s, "char"))
 		result = DDL_PrimitiveType::Char;
 	else if (iequals(s, "uchar"))
@@ -79,7 +87,16 @@ DDL_PrimitiveType map_to_primitive_type(std::string_view s)
 	else if (iequals(s, "numb"))
 		result = DDL_PrimitiveType::Numb;
 	else
-		throw validation_error("Not a known primitive type");
+		ec = make_error_code(validation_error::not_a_known_primitive_type);
+	return result;
+}
+
+DDL_PrimitiveType map_to_primitive_type(std::string_view s)
+{
+	std::error_code ec;
+	auto result = map_to_primitive_type(s, ec);
+	if (ec)
+		throw std::system_error(ec, std::string{ s });
 	return result;
 }
 
@@ -196,63 +213,72 @@ int type_validator::compare(std::string_view a, std::string_view b) const
 
 // --------------------------------------------------------------------
 
-// void ValidateItem::addLinked(ValidateItem* parent, const std::string& parentItem, const std::string& childItem)
-//{
-////	if (mParent != nullptr and VERBOSE)
-////		cerr << "replacing parent in " << mCategory->m_name << " from " << mParent->mCategory->m_name << " to " << parent->mCategory->m_name << endl;
-////	mParent = parent;
-//
-//	if (m_type == nullptr and parent != nullptr)
-//		m_type = parent->m_type;
-//
-//	if (parent != nullptr)
-//	{
-//		mLinked.push_back({parent, parentItem, childItem});
-//
-//		parent->mChildren.insert(this);
-////
-////		if (mCategory->mKeys == std::vector<std::string>{mTag})
-////			parent->mForeignKeys.insert(this);
-//	}
-//}
-
 void item_validator::operator()(std::string_view value) const
 {
+	std::error_code ec;
+	if (not validate_value(value, ec))
+		throw std::system_error(ec, std::string{ value } + " does not match rx for " + m_item_name);
+}
+
+bool item_validator::validate_value(std::string_view value, std::error_code &ec) const noexcept
+{
+	ec = {};
+
 	if (not value.empty() and value != "?" and value != ".")
 	{
 		if (m_type != nullptr and not regex_match(value.begin(), value.end(), *m_type->m_rx))
-			throw validation_error(m_category->m_name, m_tag, "Value '" + std::string{ value } + "' does not match type expression for type " + m_type->m_name);
-
-		if (not m_enums.empty())
-		{
-			if (m_enums.count(std::string{ value }) == 0)
-				throw validation_error(m_category->m_name, m_tag, "Value '" + std::string{ value } + "' is not in the list of allowed values");
-		}
+			ec = make_error_code(validation_error::value_does_not_match_rx);
+		else if (not m_enums.empty() and m_enums.count(std::string{ value }) == 0)
+			ec = make_error_code(validation_error::value_is_not_in_enumeration_list);
 	}
+
+	return ec == std::errc();
 }
 
 // --------------------------------------------------------------------
 
-void category_validator::addItemValidator(item_validator &&v)
+void category_validator::add_item_validator(item_validator &&v)
 {
 	if (v.m_mandatory)
-		m_mandatory_fields.insert(v.m_tag);
+		m_mandatory_items.insert(v.m_item_name);
 
 	v.m_category = this;
 
 	auto r = m_item_validators.insert(std::move(v));
 	if (not r.second and VERBOSE >= 4)
-		std::cout << "Could not add validator for item " << v.m_tag << " to category " << m_name << '\n';
+		std::cout << "Could not add validator for item " << v.m_item_name << " to category " << m_name << '\n';
 }
 
-const item_validator *category_validator::get_validator_for_item(std::string_view tag) const
+const item_validator *category_validator::get_validator_for_item(std::string_view item_name) const
 {
 	const item_validator *result = nullptr;
-	auto i = m_item_validators.find(item_validator{ std::string(tag) });
+	auto i = m_item_validators.find(item_validator{ std::string(item_name) });
 	if (i != m_item_validators.end())
 		result = &*i;
 	else if (VERBOSE > 4)
-		std::cout << "No validator for tag " << tag << '\n';
+		std::cout << "No validator for item " << item_name << '\n';
+	return result;
+}
+
+const item_validator *category_validator::get_validator_for_aliased_item(std::string_view item_name) const
+{
+	const item_validator *result = nullptr;
+
+	for (auto &iv : m_item_validators)
+	{
+		for (auto &ai : iv.m_aliases)
+		{
+			const auto &[cat, name] = split_item_name(ai.m_name);
+			if (iequals(name, item_name) and iequals(cat, m_name))
+			{
+				result = &iv;
+				break;
+			}
+		}
+		if (result)
+			break;
+	}
+
 	return result;
 }
 
@@ -295,19 +321,19 @@ const category_validator *validator::get_validator_for_category(std::string_view
 	return result;
 }
 
-item_validator *validator::get_validator_for_item(std::string_view tag) const
+item_validator *validator::get_validator_for_item(std::string_view item_name) const
 {
 	item_validator *result = nullptr;
 
 	std::string cat, item;
-	std::tie(cat, item) = split_tag_name(tag);
+	std::tie(cat, item) = split_item_name(item_name);
 
 	auto *cv = get_validator_for_category(cat);
 	if (cv != nullptr)
 		result = const_cast<item_validator *>(cv->get_validator_for_item(item));
 
 	if (result == nullptr and VERBOSE > 4)
-		std::cout << "No validator for item " << tag << '\n';
+		std::cout << "No validator for item " << item_name << '\n';
 
 	return result;
 }
@@ -332,11 +358,11 @@ void validator::add_link_validator(link_validator &&v)
 		auto piv = pcv->get_validator_for_item(v.m_parent_keys[i]);
 
 		if (piv == nullptr)
-			throw std::runtime_error("unknown parent tag _" + v.m_parent_category + '.' + v.m_parent_keys[i]);
+			throw std::runtime_error("unknown parent item _" + v.m_parent_category + '.' + v.m_parent_keys[i]);
 
 		auto civ = ccv->get_validator_for_item(v.m_child_keys[i]);
 		if (civ == nullptr)
-			throw std::runtime_error("unknown child tag _" + v.m_child_category + '.' + v.m_child_keys[i]);
+			throw std::runtime_error("unknown child item _" + v.m_child_category + '.' + v.m_child_keys[i]);
 
 		if (civ->m_type == nullptr and piv->m_type != nullptr)
 			const_cast<item_validator *>(civ)->m_type = piv->m_type;
@@ -351,7 +377,7 @@ std::vector<const link_validator *> validator::get_links_for_parent(std::string_
 
 	for (auto &l : m_link_validators)
 	{
-		if (l.m_parent_category == category)
+		if (iequals(l.m_parent_category, category))
 			result.push_back(&l);
 	}
 
@@ -364,19 +390,32 @@ std::vector<const link_validator *> validator::get_links_for_child(std::string_v
 
 	for (auto &l : m_link_validators)
 	{
-		if (l.m_child_category == category)
+		if (iequals(l.m_child_category, category))
 			result.push_back(&l);
 	}
 
 	return result;
 }
 
-void validator::report_error(const std::string &msg, bool fatal) const
+void validator::report_error(std::error_code ec, bool fatal) const
 {
 	if (m_strict or fatal)
-		throw validation_error(msg);
-	else if (VERBOSE > 0)
-		std::cerr << msg << '\n';
+		throw validation_exception(ec);
+	else
+		std::cerr << ec.message() << '\n';
+}
+
+void validator::report_error(std::error_code ec, std::string_view category,
+	std::string_view item, bool fatal) const
+{
+	auto ex = item.empty() ?
+		validation_exception(ec, category) :
+		validation_exception(ec, category, item);
+
+	if (m_strict or fatal)
+		throw ex;
+	else
+		std::cerr << ex.what() << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -438,12 +477,12 @@ const validator &validator_factory::operator[](std::string_view dictionary_name)
 			if (not std::filesystem::exists(p, ec) or ec)
 			{
 				for (const char *dir : {
-#if defined(CACHE_DIR)
+# if defined(CACHE_DIR)
 						 CACHE_DIR,
-#endif
-#if defined(DATA_DIR)
+# endif
+# if defined(DATA_DIR)
 							 DATA_DIR
-#endif
+# endif
 					 })
 				{
 					auto p2 = std::filesystem::path(dir) / p;
